<?php

namespace TVn7\EventBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\Common\Collections\Criteria;
use Doctrine\Common\Collections\ArrayCollection;


use TVn7\EventBundle\Form\AllFilenameType;
use TVn7\EventBundle\Form\EventType;

use TVn7\EventBundle\Entity\ClipType;
use TVn7\EventBundle\Entity\Comment;
use TVn7\EventBundle\Entity\Clip;
use TVn7\EventBundle\Entity\Event;
use TVn7\EventBundle\Entity\Master;
use TVn7\EventBundle\Entity\Music;
use TVn7\EventBundle\Entity\Sequence;
use TVn7\UserBundle\Entity\Membre;
use TVn7\EventBundle\Entity\TrackSettings;
use TVn7\EventBundle\Form\FilenameType;
use TVn7\EventBundle\Form\TrackSettingsType;

use TVn7\EventBundle\Security\EventVoter;

use Symfony\Component\Routing\Generator\UrlGenerator;

/**
 * @Security("has_role('ROLE_EVENT_USER')")
 */
class EventController extends Controller
{
    public function indexAction()
    {
        $eventsavenir = $this->getDoctrine()->getManager()->getRepository('TVn7EventBundle:Event')->findAllAVenir();
        $eventspasses = $this->getDoctrine()->getManager()->getRepository('TVn7EventBundle:Event')->findAllPasses();

        return $this->render('TVn7EventBundle:Event:index.html.twig',['eventsAVenir' => $eventsavenir, 'eventsPasses' => $eventspasses]);

    }
    
    /**
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|Response
     */
    public function newAction(Request $request)
    {
        $this->denyAccessUnlessGranted('ROLE_EVENT_ADMIN');

        $event = new Event();
        $form = $this->createForm(EventType::class, $event);

        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($event);
                $em->flush();

                $this->addFlash('success', 'L\'évenement a bien été créé.');

                return $this->redirectToRoute("_tdb_event_show", array("slug" => $event->getSlug()));
            }
        }

        return $this->render('TVn7EventBundle:Event:new.html.twig', array(
            'form' => $form->createView(),
        ));
    }

    /**
     *
     * @param Event $event
     * @param Request $request
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|Response
     */
    public function editAction(Event $event, Request $request)
    {
        $this->denyAccessUnlessGranted(EventVoter::EDIT, $event);

        $form = $this->createForm(EventType::class, $event);

        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($event);
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");

                $notifier->notify(
                        $event, "Édition d'un évènement",
                        sprintf("%s a édité l'évènement « %s »", $this->getUser(), $event->getName()),
                        $this->generateUrl("_tdb_event_show", ["slug" => $event->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'L\'évenement a bien été modifié.');

                return $this->redirectToRoute("_tdb_event_show", array("slug" => $event->getSlug()));
            }
        }

        return $this->render('TVn7EventBundle:Event:edit.html.twig', array(
            'form' => $form->createView(),
            'event' => $event,
        ));
    }

    /**
     * 
     * @param Event $event
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(Event $event)
    {
        $this->denyAccessUnlessGranted(EventVoter::DELETE, $event);

        $event->setDeleted(true);
        $this->getDoctrine()->getManager()->flush();

        $notifier = $this->get("tvn7.event_notifier");
        $notifier->notify(
                $event, "Suppression d'un évènement",
                sprintf("%s a supprimé l'évènement « %s »", $this->getUser(), $event->getName()),
                $this->generateUrl("_tdb_event_show", ["slug" => $event->getSlug()], UrlGenerator::ABSOLUTE_URL)
        );

        $this->addFlash('success', 'L\'évenement a bien été supprimé.');
        return $this->redirectToRoute("_tdb_event_index");
    }
    
    public function showAction(Event $event) {
        $this->denyAccessUnlessGranted(EventVoter::SHOW, $event);

        $clips = $event->getClips()->filter(function($c) { return $c->getDeleted() == false;});

        $musics = $this->getDoctrine()->getManager()
                ->getRepository('TVn7EventBundle:Music')
                ->findByEvent($event);

        $clipsMusicsConflict = new ArrayCollection();
        foreach($musics as $m) {
            if($m->getClips()->count() > 1) {
                foreach($m->getClips() as $c) {
                    if(!$clipsMusicsConflict->contains($c)) {
                        $clipsMusicsConflict->add($c);
                    }
                }
            }
        }
        
        return $this->render('TVn7EventBundle:Event:show.html.twig',[
            'event' => $event,
            'clipsEnCoursTournage' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_UNSHOTED;}),
            'clipsEnCoursDerush' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_SHOTED;}),
            'clipsEnCoursMontage' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_DERUSHED;}),
            'clipsMontes' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_FINISHED;}),
            'clipsValides' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_VALIDATED;}),
            'clipsAnnules' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_CANCELED;}),
            'clipsOrphelins' => $clips->filter(function($c) { return $c->getStatus() == Clip::S_ORPHAN;}),
            'clipsMusicsConflict' => $clipsMusicsConflict
        ]);
    }

    public function trackAction(Event $event, Request $request) {
        $this->denyAccessUnlessGranted(EventVoter::SHOW, $event);

        return $response = $this->render('TVn7EventBundle:Event:track.html.twig', ['event' => $event]);
    }

    public function trackUpdateAction(Event $event, Request $request) {
        $this->denyAccessUnlessGranted(EventVoter::SHOW, $event);

        if(!$request->isXmlHttpRequest()) {
            throw $this->createNotFoundException();
        }
        $clips = $event->getClips()->filter(function($c) {
            return $c->getDeleted() == false;
        });

        $ts = $event->getTrackSettings();

        $filter = [];
        if($ts->getCanceledDisplay())  $filter[] = Clip::S_CANCELED;
        if($ts->getDerushedDisplay())  $filter[] = Clip::S_DERUSHED;
        if($ts->getFinishedDisplay())  $filter[] = Clip::S_FINISHED;
        if($ts->getOrphanDisplay())    $filter[] = Clip::S_ORPHAN;
        if($ts->getShotedDisplay())    $filter[] = Clip::S_SHOTED;
        if($ts->getUnstartedDisplay()) $filter[] = Clip::S_UNSHOTED;
        if($ts->getValidatedDisplay()) $filter[] = Clip::S_VALIDATED;

        $clips_filtered = $clips->filter(function($c) use($filter) {
            return in_array($c->getStatus(), $filter);
        });

        $criteria = Criteria::create();

        switch($ts->getOrder()) {
            case TrackSettings::ORDER_DATE_ASC:
                $criteria->orderBy(array("createdAt" => Criteria::ASC));
                break;
            case TrackSettings::ORDER_DATE_DESC:
                $criteria->orderBy(array("createdAt" => Criteria::DESC));
                break;
            case TrackSettings::ORDER_ALPHA_ASC:
                $criteria->orderBy(array("nameCanonical" => Criteria::ASC));
                break;
            case TrackSettings::ORDER_ALPHA_DESC:
                $criteria->orderBy(array("nameCanonical" => Criteria::DESC));
                break;
            case TrackSettings::ORDER_STATUS_ASC:
                $criteria->orderBy(array("statusInt" => Criteria::ASC));
                break;
            case TrackSettings::ORDER_STATUS_DESC:
                $criteria->orderBy(array("statusInt" => Criteria::DESC));
                break;
            default:
                throw new \Exception("Unknown order: " . $ts->getOrder());
        }

        $clips_sorted = $clips_filtered->matching($criteria);

        $data = [
            'clips_html' => $this->renderView('TVn7EventBundle:Event:trackTile.html.twig', [
                'clips' => $clips_sorted,
                'settings' => $ts
                    ]),
            'message' => $ts->getMessage(),
            'event_date' => ($event->getDate() instanceof \DateTime ? $event->getDate()->getTimestamp() : null),
            'event_deadline' => ($event->getDeadline() instanceof \DateTime ? $event->getDeadline()->getTimestamp() : null),
            'darkmode' => $ts->getDarkMode()
                ];

        return new JsonResponse($data, Response::HTTP_OK);
    }

    public function trackSettingsAction(Event $event, Request $request){
        $this->denyAccessUnlessGranted(EventVoter::EDIT, $event);

        $form = $this->createForm(TrackSettingsType::class, $event->getTrackSettings());

        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($event->getTrackSettings());
                $em->flush();

                $this->addFlash('success', 'Les paramètres du tableau virtuel ont bien été modifiés.');

                return $this->redirectToRoute("_tdb_event_show", array("slug" => $event->getSlug()));
            }
        }
        
        return $this->render('TVn7EventBundle:Event:trackSettingsForm.html.twig', array(
            'form' => $form->createView(),
            'event' => $event
        ));
    }

    public function filenameAction(Event $event, Request $request){
        $this->denyAccessUnlessGranted(EventVoter::EDIT, $event);

        $form = $this->createForm(AllFilenameType::class, $event, ['event' => $event]);

        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $em = $this->getDoctrine()->getManager();
                $em->persist($event);
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");

                $notifier->notify(
                    $event, "Édition d'un évènement",
                    sprintf("%s a édité l'évènement « %s »", $this->getUser(), $event->getName()),
                    $this->generateUrl("_tdb_event_show", ["slug" => $event->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'L\'évenement a bien été modifié.');

                return $this->redirectToRoute("_tdb_event_show", array("slug" => $event->getSlug()));
            }
        }
        $framerate = $this->getParameter("tvn7_event.config")["framerate"];
        $codec = $this->getParameter("tvn7_event.config")["codec"];
        $width = $this->getParameter("tvn7_event.config")["video_width"];
        $height = $this->getParameter("tvn7_event.config")["video_height"];
        return $this->render('TVn7EventBundle:Event:form_filename.html.twig', array(
            'form' => $form->createView(),
            'event' => $event,
            'framerate' =>$framerate,
            'codec' =>$codec,
            'width' =>$width,
            'height' =>$height,
        ));
    }

    public function musicsAction(Event $event)
    {
        $musics = $this->getDoctrine()->getManager()
                ->getRepository('TVn7EventBundle:Music')
                ->findByEvent($event);

        return $this->render('TVn7EventBundle:Event:musics.html.twig',
            [
                'musics'=>$musics, 
                'event' => $event
            ]);
    }
}
